#include "tmd3702.h"

#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"

#include "app_util_platform.h"
#include "nrf_twi_mngr.h"
#include "nrf_assert.h"
#include <stdint.h>

typedef volatile struct
{
    bool    transaction_in_progress;
    uint8_t transaction_result;
} nrf_twi_mngr_cb_data_t;

static void internal_transaction_cb(ret_code_t result, void * p_user_data)
{
}
void nrf_twi_mngr_sched_nowait(nrf_twi_mngr_t const *          p_nrf_twi_mngr,
                                nrf_drv_twi_config_t const *    p_config,
                                nrf_twi_mngr_transfer_t const * p_transfers,
                                uint8_t                         number_of_transfers,
                                void                            (* user_function)(void))
{
    ASSERT(p_nrf_twi_mngr != NULL);
    ASSERT(p_transfers != NULL);
    ASSERT(number_of_transfers != 0);

    // TODO: should this static struct be declared outsite of this function because the TWI doesn't perform at once.
    static nrf_twi_mngr_transaction_t internal_transaction =
    {
        .callback            = internal_transaction_cb,
        .p_user_data         = NULL,
        .p_transfers         = NULL,
        .number_of_transfers = 0,
        .p_required_twi_cfg  = NULL
    };
    internal_transaction.p_transfers = p_transfers;
    internal_transaction.number_of_transfers = number_of_transfers;
    internal_transaction.p_required_twi_cfg = p_config;

    ret_code_t result = nrf_twi_mngr_schedule(p_nrf_twi_mngr, &internal_transaction);
    VERIFY_SUCCESS(result);

}

int tmd3702_init(nrf_twi_mngr_t *twi) {
  
    /* reset TMD3702 */
    uint8_t x = 1;
    tmd3702_write_reg(twi, TMD_SOFTRST, 1, &x);
    //tmd3702_write_reg(twi, TMD_SOFTRST, x);

   // enable interrupts
   x = TMD_INTENAB_PIEN;//TMD_INTENAB_AIEN | TMD_INTENAB_PIEN;
   tmd3702_write_reg(twi, TMD_INTENAB, 1, &x);
   
   // enable sleep after interrupt and clear status on read of status
   x = TMD_CFG3_INT_READ_CLEAR | TMD_CFG3_SAI;
   tmd3702_write_reg(twi, TMD_CFG3, 1, &x);

    // set the integration time to 178 ms
    //x = 0x3F; // MAX RESOLUTION
    //x = 0x00; // 2.78 ms
    x = 0x0; 
    tmd3702_write_reg(twi, TMD_ATIME, 1, &x);

    //x = 60;
    tmd3702_sched_write_reg(twi, TMD_WTIME, x);

    /* enable ALS and oscillator */
    x = TMD_ENABLE_PON | TMD_ENABLE_PEN | TMD_ENABLE_AEN;
    tmd3702_sched_write_reg(twi, TMD_ENABLE, x);

    NRF_LOG_INFO("TMD initialized\n");
   for(int i = 0; i < 30; i++) {
      uint8_t tmp;
      tmd3702_read_reg(twi, 0x80 + i, 1, &tmp);
      NRF_LOG_INFO("TMD %hhx = %hhx", 0x80 + i, tmp);
   }
   return 0;
}

int tmd3702_read_reg(nrf_twi_mngr_t *twi, uint8_t addr, int n, uint8_t *data) {
    nrf_twi_mngr_transfer_t const read_tranfsers[] = {
        NRF_TWI_MNGR_WRITE(0x49, &addr, 1, NRF_TWI_MNGR_NO_STOP), 
        NRF_TWI_MNGR_READ (0x49, data, n, 0)        
    };

  
  int err = nrf_twi_mngr_perform(twi, NULL, read_tranfsers, 2, NULL);
  return err;
     
}

int tmd3702_write_reg(nrf_twi_mngr_t *twi, uint8_t addr, int n, uint8_t *data) {
    uint8_t out[30];
    out[0] = addr;
    for(int i = 0; i < n; i++)
      out[i + 1] = data[i];

    nrf_twi_mngr_transfer_t const write_tranfsers[] = {
        NRF_TWI_MNGR_WRITE(0x49, &out, n + 1, 0)
    };

  int err = nrf_twi_mngr_perform(twi, NULL, write_tranfsers, 1, NULL);
  return err;
}
int tmd3702_write_reg_nowait(nrf_twi_mngr_t *twi, uint8_t addr, int n, uint8_t *data) {
    if(n > 33)
        return NRF_ERROR_INVALID_PARAM;
    uint8_t out[33];
    out[0] = addr;
    for(int i = 0; i < n; i++)
      out[i + 1] = data[i];

    nrf_twi_mngr_transfer_t const write_tranfsers[] = {
        NRF_TWI_MNGR_WRITE(0x49, &out, n + 1, 0)
    };

  //int err = nrf_twi_mngr_perform(twi, NULL, write_tranfsers, 1, NULL);
  nrf_twi_mngr_sched_nowait(twi, NULL, write_tranfsers, 1, NULL);
  return NRF_SUCCESS;
}
/* schedule to read color data and to read status from TMD3702 */
int tmd3702_sched_read_sensor(nrf_twi_mngr_t *twi, nrf_twi_mngr_callback_t cb) {
    static uint8_t color_addr = TMD_CDATAL;
    static uint8_t status_addr = TMD_STATUS;

    static nrf_twi_mngr_transfer_t  const read_tranfsers[] = {
        NRF_TWI_MNGR_WRITE(0x49, &color_addr, 1, NRF_TWI_MNGR_NO_STOP), 
        NRF_TWI_MNGR_READ (0x49, color_data, 10, 0),
        NRF_TWI_MNGR_WRITE(0x49, &status_addr, 1, NRF_TWI_MNGR_NO_STOP), 
        NRF_TWI_MNGR_READ (0x49, &tmd_status, 1, 0)
    };
    static nrf_twi_mngr_transaction_t internal_transaction =
    {
        .callback            = NULL,
        .p_user_data         = color_data,
        .p_transfers         = read_tranfsers,
        .number_of_transfers = 4,
        .p_required_twi_cfg  = NULL
    };
    internal_transaction.callback = cb;

    ret_code_t result = nrf_twi_mngr_schedule(twi, &internal_transaction);
    VERIFY_SUCCESS(result);
}

int tmd3702_sched_write_reg(nrf_twi_mngr_t *twi, uint8_t addr, uint8_t data) {
    static uint8_t out[2];
     out[0] = addr;
     out[1] = data;

    static nrf_twi_mngr_transfer_t const write_tranfsers[] = {
        NRF_TWI_MNGR_WRITE(0x49, &out, 2, 0)
    };
    static nrf_twi_mngr_transaction_t internal_transaction =
    {
        .callback            = NULL,
        .p_user_data         = out,
        .p_transfers         = write_tranfsers,
        .number_of_transfers = 1,
        .p_required_twi_cfg  = NULL
    };
    ret_code_t result = nrf_twi_mngr_schedule(twi, &internal_transaction);
    VERIFY_SUCCESS(result);
}

int tmd3702_sched_read_reg(nrf_twi_mngr_t *twi, uint8_t addr, uint8_t n, nrf_twi_mngr_callback_t cb) {
    static uint8_t read_addr;
    static uint8_t status_addr = TMD_STATUS;

    static nrf_twi_mngr_transfer_t   read_tranfsers[] = {
        NRF_TWI_MNGR_WRITE(0x49, &read_addr, 1, NRF_TWI_MNGR_NO_STOP), 
        NRF_TWI_MNGR_READ (0x49, read_data, 0, 0)
    };

    read_addr = addr;
    read_tranfsers[1].length = n;

    static nrf_twi_mngr_transaction_t internal_transaction =
    {
        .callback            = NULL,
        .p_user_data         = read_data,
        .p_transfers         = read_tranfsers,
        .number_of_transfers = 2,
        .p_required_twi_cfg  = NULL
    };
    internal_transaction.callback = cb;

    ret_code_t result = nrf_twi_mngr_schedule(twi, &internal_transaction);
    VERIFY_SUCCESS(result);
}